@@ -1,5 +1,5 @@
 libDir = node.path.join(node.path.dirname(__filename), "../lib");
-node.libraryPaths.unshift(libDir);
+require.paths.unshift(libDir);
 http = require("/http.js");
 var concurrency = 30;
 var port = 8000;
