@@ -38,7 +38,7 @@ struct message {
 static struct message messages[5];
 static int num_messages;
 
-/* * R E Q U E S T S * */ 
+/* * R E Q U E S T S * */
 const struct message requests[] =
 #define CURL_GET 0
 { {.name= "curl get"
@@ -55,7 +55,7 @@ const struct message requests[] =
   ,.request_path= "/test"
   ,.request_uri= "/test"
   ,.num_headers= 3
-  ,.headers= 
+  ,.headers=
     { { "User-Agent", "curl/7.18.0 (i486-pc-linux-gnu) libcurl/7.18.0 OpenSSL/0.9.8g zlib/1.2.3.3 libidn/1.1" }
     , { "Host", "0.0.0.0=5000" }
     , { "Accept", "*/*" }
@@ -83,7 +83,7 @@ const struct message requests[] =
   ,.request_path= "/favicon.ico"
   ,.request_uri= "/favicon.ico"
   ,.num_headers= 8
-  ,.headers= 
+  ,.headers=
     { { "Host", "0.0.0.0=5000" }
     , { "User-Agent", "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9) Gecko/2008061015 Firefox/3.0" }
     , { "Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8" }
@@ -109,7 +109,7 @@ const struct message requests[] =
   ,.request_path= "/dumbfuck"
   ,.request_uri= "/dumbfuck"
   ,.num_headers= 1
-  ,.headers= 
+  ,.headers=
     { { "aaaaaaaaaaaaa",  "++++++++++" }
     }
   ,.body= ""
@@ -126,7 +126,7 @@ const struct message requests[] =
   ,.fragment= "posts-17408"
   ,.request_path= "/forums/1/topics/2375"
   /* XXX request uri does not include fragment? */
-  ,.request_uri= "/forums/1/topics/2375?page=1" 
+  ,.request_uri= "/forums/1/topics/2375?page=1"
   ,.num_headers= 0
   ,.body= ""
   }
@@ -159,7 +159,7 @@ const struct message requests[] =
   ,.request_path= "/get_one_header_no_body"
   ,.request_uri= "/get_one_header_no_body"
   ,.num_headers= 1
-  ,.headers= 
+  ,.headers=
     { { "Accept" , "*/*" }
     }
   ,.body= ""
@@ -179,7 +179,7 @@ const struct message requests[] =
   ,.request_path= "/get_funky_content_length_body_hello"
   ,.request_uri= "/get_funky_content_length_body_hello"
   ,.num_headers= 1
-  ,.headers= 
+  ,.headers=
     { { "conTENT-Length" , "5" }
     }
   ,.body= "HELLO"
@@ -201,10 +201,10 @@ const struct message requests[] =
   ,.request_path= "/post_identity_body_world"
   ,.request_uri= "/post_identity_body_world?q=search"
   ,.num_headers= 3
-  ,.headers= 
+  ,.headers=
     { { "Accept", "*/*" }
     , { "Transfer-Encoding", "identity" }
-    , { "Content-Length", "5" } 
+    , { "Content-Length", "5" }
     }
   ,.body= "World"
   }
@@ -225,7 +225,7 @@ const struct message requests[] =
   ,.request_path= "/post_chunked_all_your_base"
   ,.request_uri= "/post_chunked_all_your_base"
   ,.num_headers= 1
-  ,.headers= 
+  ,.headers=
     { { "Transfer-Encoding" , "chunked" }
     }
   ,.body= "all your base are belong to us"
@@ -248,13 +248,13 @@ const struct message requests[] =
   ,.request_path= "/two_chunks_mult_zero_end"
   ,.request_uri= "/two_chunks_mult_zero_end"
   ,.num_headers= 1
-  ,.headers= 
+  ,.headers=
     { { "Transfer-Encoding", "chunked" }
     }
   ,.body= "hello world"
   }
 
-#define CHUNKED_W_TRAILING_HEADERS 10 
+#define CHUNKED_W_TRAILING_HEADERS 10
 , {.name= "chunked with trailing headers. blech."
   ,.type= HTTP_REQUEST
   ,.raw= "POST /chunked_w_trailing_headers HTTP/1.1\r\n"
@@ -273,13 +273,13 @@ const struct message requests[] =
   ,.request_path= "/chunked_w_trailing_headers"
   ,.request_uri= "/chunked_w_trailing_headers"
   ,.num_headers= 1
-  ,.headers= 
+  ,.headers=
     { { "Transfer-Encoding",  "chunked" }
     }
   ,.body= "hello world"
   }
 
-#define CHUNKED_W_BULLSHIT_AFTER_LENGTH 11 
+#define CHUNKED_W_BULLSHIT_AFTER_LENGTH 11
 , {.name= "with bullshit after the length"
   ,.type= HTTP_REQUEST
   ,.raw= "POST /chunked_w_bullshit_after_length HTTP/1.1\r\n"
@@ -296,17 +296,32 @@ const struct message requests[] =
   ,.request_path= "/chunked_w_bullshit_after_length"
   ,.request_uri= "/chunked_w_bullshit_after_length"
   ,.num_headers= 1
-  ,.headers= 
+  ,.headers=
     { { "Transfer-Encoding", "chunked" }
     }
   ,.body= "hello world"
   }
 
+#define WITH_QUOTES 12
+, {.name= "with quotes"
+  ,.type= HTTP_REQUEST
+  ,.raw= "GET /with_\"stupid\"_quotes?foo=\"bar\" HTTP/1.1\r\n\r\n"
+  ,.should_keep_alive= TRUE
+  ,.method= HTTP_GET
+  ,.query_string= "foo=\"bar\""
+  ,.fragment= ""
+  ,.request_path= "/with_\"stupid\"_quotes"
+  ,.request_uri= "/with_\"stupid\"_quotes?foo=\"bar\""
+  ,.num_headers= 0
+  ,.headers= { }
+  ,.body= ""
+  }
+
 , {.name= NULL } /* sentinel */
 };
 
-/* * R E S P O N S E S * */ 
-const struct message responses[] = 
+/* * R E S P O N S E S * */
+const struct message responses[] =
 { {.name= "google 301"
   ,.type= HTTP_RESPONSE
   ,.raw= "HTTP/1.1 301 Moved Permanently\r\n"
@@ -327,7 +342,7 @@ const struct message responses[] =
   ,.should_keep_alive= TRUE
   ,.status_code= 301
   ,.num_headers= 7
-  ,.headers= 
+  ,.headers=
     { { "Location", "http://www.google.com/" }
     , { "Content-Type", "text/html; charset=UTF-8" }
     , { "Date", "Sun, 26 Apr 2009 11:11:49 GMT" }
@@ -344,6 +359,45 @@ const struct message responses[] =
           "</BODY></HTML>\r\n"
   }
 
+, {.name= "no content-length response"
+  ,.type= HTTP_RESPONSE
+  ,.raw= "HTTP/1.1 200 OK\r\n"
+         "Date: Tue, 04 Aug 2009 07:59:32 GMT\r\n"
+         "Server: Apache\r\n"
+         "X-Powered-By: Servlet/2.5 JSP/2.1\r\n"
+         "Content-Type: text/xml; charset=utf-8\r\n"
+         "Connection: close\r\n"
+         "\r\n"
+         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+         "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\">\n"
+         "  <SOAP-ENV:Body>\n"
+         "    <SOAP-ENV:Fault>\n"
+         "       <faultcode>SOAP-ENV:Client</faultcode>\n"
+         "       <faultstring>Client Error</faultstring>\n"
+         "    </SOAP-ENV:Fault>\n"
+         "  </SOAP-ENV:Body>\n"
+         "</SOAP-ENV:Envelope>"
+  ,.should_keep_alive= FALSE
+  ,.status_code= 200
+  ,.num_headers= 5
+  ,.headers=
+    { { "Date", "Tue, 04 Aug 2009 07:59:32 GMT" }
+    , { "Server", "Apache" }
+    , { "X-Powered-By", "Servlet/2.5 JSP/2.1" }
+    , { "Content-Type", "text/xml; charset=utf-8" }
+    , { "Connection", "close" }
+    }
+  ,.body= "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+          "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\">\n"
+          "  <SOAP-ENV:Body>\n"
+          "    <SOAP-ENV:Fault>\n"
+          "       <faultcode>SOAP-ENV:Client</faultcode>\n"
+          "       <faultstring>Client Error</faultstring>\n"
+          "    </SOAP-ENV:Fault>\n"
+          "  </SOAP-ENV:Body>\n"
+          "</SOAP-ENV:Envelope>"
+  }
+
 , {.name= "404 no headers no body"
   ,.type= HTTP_RESPONSE
   ,.raw= "HTTP/1.1 404 Not Found\r\n\r\n"
@@ -362,7 +416,34 @@ const struct message responses[] =
   ,.num_headers= 0
   ,.headers= {}
   ,.body= ""
-}
+  }
+
+, {.name="200 trailing space on chunked body"
+  ,.type= HTTP_RESPONSE
+  ,.raw= "HTTP/1.1 200 OK\r\n"
+         "Content-Type: text/plain\r\n"
+         "Transfer-Encoding: chunked\r\n"
+         "\r\n"
+         "25  \r\n"
+         "This is the data in the first chunk\r\n"
+         "\r\n"
+         "1C\r\n"
+         "and this is the second one\r\n"
+         "\r\n"
+         "0  \r\n"
+         "\r\n"
+  ,.should_keep_alive= TRUE
+  ,.status_code= 200
+  ,.num_headers= 2
+  ,.headers=
+    { {"Content-Type", "text/plain" }
+    , {"Transfer-Encoding", "chunked" }
+    }
+  ,.body =
+         "This is the data in the first chunk\r\n"
+         "and this is the second one\r\n"
+
+  }
 
 , {.name= NULL } /* sentinel */
 };
@@ -528,12 +609,14 @@ parse_messages (int message_count, const struct message *input_messages[])
   }
 
   // Parse the stream
-  size_t traversed = 0;
   parser_init(HTTP_REQUEST);
 
-  traversed = http_parser_execute(&parser, total, length);
-
+  http_parser_execute(&parser, total, length);
   assert(!http_parser_has_error(&parser));
+
+  http_parser_execute(&parser, NULL, 0);
+  assert(!http_parser_has_error(&parser));
+
   assert(num_messages == message_count);
 
   for (i = 0; i < message_count; i++) {
@@ -545,11 +628,14 @@ parse_messages (int message_count, const struct message *input_messages[])
 void
 test_message (const struct message *message)
 {
-  size_t traversed = 0;
   parser_init(message->type);
 
-  traversed = http_parser_execute(&parser, message->raw, strlen(message->raw));
+  http_parser_execute(&parser, message->raw, strlen(message->raw));
   assert(!http_parser_has_error(&parser));
+
+  http_parser_execute(&parser, NULL, 0);
+  assert(!http_parser_has_error(&parser));
+
   assert(num_messages == 1);
 
   message_eq(0, message);
@@ -558,10 +644,10 @@ test_message (const struct message *message)
 void
 test_error (const char *buf)
 {
-  size_t traversed = 0;
   parser_init(HTTP_REQUEST);
 
-  traversed = http_parser_execute(&parser, buf, strlen(buf));
+  http_parser_execute(&parser, buf, strlen(buf));
+  http_parser_execute(&parser, NULL, 0);
 
   assert(http_parser_has_error(&parser));
 }
@@ -569,30 +655,32 @@ test_error (const char *buf)
 void
 test_multiple3 (const struct message *r1, const struct message *r2, const struct message *r3)
 {
-  char total[ strlen(r1->raw) 
-            + strlen(r2->raw) 
-            + strlen(r3->raw) 
+  char total[ strlen(r1->raw)
+            + strlen(r2->raw)
+            + strlen(r3->raw)
             + 1
             ];
   total[0] = '\0';
 
-  strcat(total, r1->raw); 
-  strcat(total, r2->raw); 
-  strcat(total, r3->raw); 
+  strcat(total, r1->raw);
+  strcat(total, r2->raw);
+  strcat(total, r3->raw);
 
-  size_t traversed = 0;
   parser_init(HTTP_REQUEST);
 
-  traversed = http_parser_execute(&parser, total, strlen(total));
+  http_parser_execute(&parser, total, strlen(total));
+  assert(!http_parser_has_error(&parser) );
+
+  http_parser_execute(&parser, NULL, 0);
+  assert(!http_parser_has_error(&parser) );
 
-  assert(! http_parser_has_error(&parser) );
   assert(num_messages == 3);
   message_eq(0, r1);
   message_eq(1, r2);
   message_eq(2, r3);
 }
 
-/* SCAN through every possible breaking to make sure the 
+/* SCAN through every possible breaking to make sure the
  * parser can handle getting the content in any chunks that
  * might come from the socket
  */
@@ -604,13 +692,13 @@ test_scan (const struct message *r1, const struct message *r2, const struct mess
   char buf2[80*1024] = "\0";
   char buf3[80*1024] = "\0";
 
-  strcat(total, r1->raw); 
-  strcat(total, r2->raw); 
-  strcat(total, r3->raw); 
+  strcat(total, r1->raw);
+  strcat(total, r2->raw);
+  strcat(total, r3->raw);
 
   int total_len = strlen(total);
 
-  int total_ops = (total_len - 1) * (total_len - 2) / 2; 
+  int total_ops = (total_len - 1) * (total_len - 2) / 2;
   int ops = 0 ;
 
   int i,j;
@@ -644,16 +732,16 @@ test_scan (const struct message *r1, const struct message *r2, const struct mess
       */
 
       http_parser_execute(&parser, buf1, buf1_len);
-
       assert(!http_parser_has_error(&parser));
 
       http_parser_execute(&parser, buf2, buf2_len);
-
       assert(!http_parser_has_error(&parser));
 
       http_parser_execute(&parser, buf3, buf3_len);
+      assert(!http_parser_has_error(&parser));
 
-      assert(! http_parser_has_error(&parser));
+      http_parser_execute(&parser, NULL, 0);
+      assert(!http_parser_has_error(&parser));
 
       assert(3 == num_messages);
 
@@ -672,14 +760,14 @@ main (void)
 
   printf("sizeof(http_parser) = %d\n", sizeof(http_parser));
 
-  int request_count; 
+  int request_count;
   for (request_count = 0; requests[request_count].name; request_count++);
 
-  int response_count; 
+  int response_count;
   for (response_count = 0; responses[response_count].name; response_count++);
 
 
-  //// RESPONSES  
+  //// RESPONSES
 
   for (i = 0; i < response_count; i++) {
     test_message(&responses[i]);
@@ -740,7 +828,7 @@ main (void)
                                            "Accept: */*\r\n"
                                            "\r\n"
                                            "HELLO";
-  test_error(bad_get_no_headers_no_body); 
+  test_error(bad_get_no_headers_no_body);
 
 
   /* TODO sending junk and large headers gets rejected */
