@@ -1,4 +1,4 @@
-var utils = require("/utils.js");
+var sys = require("/sys.js");
 
 var CRLF = "\r\n";
 var STATUS_CODES = {
@@ -333,7 +333,7 @@ function ClientRequest (method, uri, headers) {
   }
   this.closeOnFinish = true;
 
-  this.sendHeaderLines(method + " " + uri + " HTTP/1.1\r\n", headers);
+  this.sendHeaderLines(method + " " + uri + " HTTP/1.0\r\n", headers);
 }
 node.inherits(ClientRequest, OutgoingMessage);
 
@@ -476,7 +476,6 @@ function connectionListener (connection) {
     }
   });
 
-
   createIncomingMessageStream(connection, function (incoming, should_keep_alive) {
     var req = incoming;
 
@@ -502,12 +501,14 @@ exports.createClient = function (port, host) {
   client._pushRequest = function (req) {
     req.addListener("flush", function () {
       if (client.readyState == "closed") {
-        //utils.debug("HTTP CLIENT request flush. reconnect.  readyState = " + client.readyState);
+        //sys.debug("HTTP CLIENT request flush. reconnect.  readyState = " + client.readyState);
         client.connect(port, host); // reconnect
         return;
       }
-      //utils.debug("client flush  readyState = " + client.readyState);
-      if (req == requests[0]) flushMessageQueue(client, [req]);
+      //sys.debug("client flush  readyState = " + client.readyState);
+      if (req == requests[0]) {
+        if(flushMessageQueue(client, [req])) client.close();
+      }
     });
     requests.push(req);
   };
@@ -517,7 +518,7 @@ exports.createClient = function (port, host) {
   });
 
   client.addListener("eof", function () {
-    //utils.debug("client got eof closing. readyState = " + client.readyState);
+    //sys.debug("client got eof closing. readyState = " + client.readyState);
     client.close();
   });
 
@@ -527,20 +528,20 @@ exports.createClient = function (port, host) {
       return;
     }
 
-    //utils.debug("HTTP CLIENT onClose. readyState = " + client.readyState);
+    //sys.debug("HTTP CLIENT onClose. readyState = " + client.readyState);
 
     // If there are more requests to handle, reconnect.
     if (requests.length > 0 && client.readyState != "opening") {
-      //utils.debug("HTTP CLIENT: reconnecting readyState = " + client.readyState);
+      //sys.debug("HTTP CLIENT: reconnecting readyState = " + client.readyState);
       client.connect(port, host); // reconnect
     }
   });
 
   createIncomingMessageStream(client, function (res) {
-   //utils.debug("incoming response!");
+   //sys.debug("incoming response!");
 
     res.addListener("complete", function ( ) {
-      //utils.debug("request complete disconnecting. readyState = " + client.readyState);
+      //sys.debug("request complete disconnecting. readyState = " + client.readyState);
       client.close();
     });
 
