@@ -1,9 +1,9 @@
 // A repl library that you can include in your own code to get a runtime
 // interface to your program. Just require("/repl.js").
 
-var utils = require("utils.js");
+var sys = require("/sys.js");
 
-utils.puts("Type '.help' for options.");
+sys.puts("Type '.help' for options.");
 
 
 var buffered_cmd = '';
@@ -49,7 +49,7 @@ function readline (cmd) {
         var ret = eval(buffered_cmd);
         if (ret !== undefined) {
           exports.scope['_'] = ret;
-          utils.p(ret);
+          sys.p(ret);
         }
       }
         
@@ -60,9 +60,9 @@ function readline (cmd) {
   } catch (e) {
     // On error: Print the error and clear the buffer
     if (e.stack) {
-      utils.puts(e.stack);
+      sys.puts(e.stack);
     } else {
-      utils.puts(e.toString());
+      sys.puts(e.toString());
     }
     buffered_cmd = '';
   }
@@ -75,7 +75,7 @@ function readline (cmd) {
  * Used to display the prompt. 
  */
 function displayPrompt () {
-  utils.print(buffered_cmd.length ? '...   ' : exports.prompt);
+  sys.print(buffered_cmd.length ? '...   ' : exports.prompt);
 }
 
 /**
@@ -91,7 +91,7 @@ function parseREPLKeyword (cmd) {
     displayPrompt();
     return true;
   case ".clear":
-    utils.puts("Clearing Scope...");
+    sys.puts("Clearing Scope...");
     buffered_cmd = '';
     exports.scope = {};
     displayPrompt();
@@ -100,10 +100,10 @@ function parseREPLKeyword (cmd) {
     node.stdio.close();
     return true;
   case ".help":
-    utils.puts(".break\tSometimes you get stuck in a place you can't get out... This will get you out.");
-    utils.puts(".clear\tBreak, and also clear the local scope.");
-    utils.puts(".exit\tExit the prompt");
-    utils.puts(".help\tShow repl options");
+    sys.puts(".break\tSometimes you get stuck in a place you can't get out... This will get you out.");
+    sys.puts(".clear\tBreak, and also clear the local scope.");
+    sys.puts(".exit\tExit the prompt");
+    sys.puts(".help\tShow repl options");
     displayPrompt();
     return true;
   }
