@@ -20,7 +20,23 @@ node.EventEmitter.prototype.listeners = function (type) {
   return this._events[type];
 };
 
-// node.Promise is defined in src/events.cc
+node.Promise.prototype.timeout = function(timeout) {
+  if (timeout === undefined) {
+    return this._timeoutDuration;
+  }
+
+  this._timeoutDuration = timeout;
+  if (this._timer) {
+    clearTimeout(this._timer);
+  }
+
+  var self = this
+  this._timer = setTimeout(function() {
+    self.emitError(new Error('timeout'));
+  }, this._timeoutDuration);
+
+  return this;
+};
 
 node.Promise.prototype.addCallback = function (listener) {
   this.addListener("success", listener);
