@@ -24,7 +24,7 @@ node.createChildProcess = function (file, args, env) {
 };
 
 node.exec = function () {
-  throw new Error("node.exec() has moved. Use require('/utils.js') to bring it back.");
+  throw new Error("node.exec() has moved. Use require('/sys.js') to bring it back.");
 }
 
 node.http.createServer = function () {
@@ -98,6 +98,25 @@ node.mixin = function() {
 	return target;
 };
 
+// Signal Handlers
+
+(function () { // anonymous namespace
+
+  function isSignal (event) {
+    return event.slice(0, 3) === 'SIG' && node.hasOwnProperty(event);
+  };
+
+  process.addListener("newListener", function (event) {
+    if (isSignal(event) && process.listeners(event).length === 0) {
+      var handler = new node.SignalHandler(node[event]);
+      handler.addListener("signal", function () {
+        process.emit(event);
+      });
+    }
+  });
+
+})(); // anonymous namespace
+
 // Timers
 
 function setTimeout (callback, after) {
@@ -275,6 +294,7 @@ node.Module.prototype.loadScript = function (loadPromise) {
       return requireAsync(url).wait();
     }
 
+    require.paths = node.libraryPaths;
     require.async = requireAsync;
 
     // create wrapper function
@@ -320,7 +340,7 @@ process.exit = function (code) {
 };
 
 node.exit = function (code) {
-  throw new Error("process.exit() has been renamed to process.exit().");
+  throw new Error("node.exit() has been renamed to process.exit().");
 };
 
 (function () {
