@@ -1,5 +1,7 @@
 var d = require("d.js");
 
+debug("load fixtures/b/c.js");
+
 var string = "C";
 
 exports.C = function () {
@@ -12,4 +14,5 @@ exports.D = function () {
 
 process.addListener("exit", function () {
   string = "C done";
+  puts("b/c.js exit");
 });
