@@ -1,4 +1,5 @@
-include("mjsunit.js");
+node.mixin(require("common.js"));
+http = require("/http.js");
 PORT = 8888;
 
 var responses_sent = 0;
@@ -6,17 +7,17 @@ var responses_recvd = 0;
 var body0 = "";
 var body1 = "";
 
-node.http.createServer(function (req, res) {
+http.createServer(function (req, res) {
   if (responses_sent == 0) {
     assertEquals("GET", req.method);
     assertEquals("/hello", req.uri.path);
 
     p(req.headers);
-    assertTrue("Accept" in req.headers);
-    assertEquals("*/*", req.headers["Accept"]);
+    assertTrue("accept" in req.headers);
+    assertEquals("*/*", req.headers["accept"]);
 
-    assertTrue("Foo" in req.headers);
-    assertEquals("bar", req.headers["Foo"]);
+    assertTrue("foo" in req.headers);
+    assertEquals("bar", req.headers["foo"]);
   }
 
   if (responses_sent == 1) {
@@ -35,14 +36,14 @@ node.http.createServer(function (req, res) {
   //assertEquals("127.0.0.1", res.connection.remoteAddress);
 }).listen(PORT);
 
-var client = node.http.createClient(PORT);
+var client = http.createClient(PORT);
 var req = client.get("/hello", {"Accept": "*/*", "Foo": "bar"});
 req.finish(function (res) {
   assertEquals(200, res.statusCode);
   responses_recvd += 1;
   res.setBodyEncoding("ascii");
   res.addListener("body", function (chunk) { body0 += chunk; });
-  node.debug("Got /hello response");
+  debug("Got /hello response");
 });
 
 setTimeout(function () {
@@ -52,15 +53,15 @@ setTimeout(function () {
     responses_recvd += 1;
     res.setBodyEncoding("utf8");
     res.addListener("body", function (chunk) { body1 += chunk; });
-    node.debug("Got /world response");
+    debug("Got /world response");
   });
 }, 1);
 
 process.addListener("exit", function () {
-  node.debug("responses_recvd: " + responses_recvd);
+  debug("responses_recvd: " + responses_recvd);
   assertEquals(2, responses_recvd);
 
-  node.debug("responses_sent: " + responses_sent);
+  debug("responses_sent: " + responses_sent);
   assertEquals(2, responses_sent);
 
   assertEquals("The path was /hello", body0);
