@@ -1,4 +1,4 @@
-include("mjsunit.js");
+node.mixin(require("common.js"));
 
 var pwd_called = false;
 
