@@ -1,18 +1,24 @@
-include("mjsunit.js");
+node.mixin(require("common.js"));
 
 var N = 40;
 var finished = false;
 
 function spawn (i) {
-  var child = node.createChildProcess('python -c "print 500 * 1024 * \'C\'"');
+  var child = node.createChildProcess( 'python'
+                                     , ['-c', 'print 500 * 1024 * "C"']
+                                     );
   var output = "";
 
   child.addListener("output", function(chunk) {
     if (chunk) output += chunk;
   });
 
+  child.addListener("error", function(chunk) {
+    if (chunk) error(chunk)
+  });
+
   child.addListener("exit", function () {
-    //puts(output);
+    puts(output);
     if (i < N)
       spawn(i+1);
     else
