@@ -1,17 +1,13 @@
-
-include("mjsunit.js");
-
-var dirname = node.path.dirname(__filename);
-var fixtures = node.path.join(dirname, "fixtures");
+node.mixin(require("common.js"));
 
 var got_error = false;
 
-var promise = node.fs.readdir(fixtures);
-puts("readdir " + fixtures);
+var promise = node.fs.readdir(fixturesDir);
+puts("readdir " + fixturesDir);
 
 promise.addCallback(function (files) {
   p(files);
-  assertArrayEquals(["a.js", "b","x.txt"], files.sort());
+  assertArrayEquals(["a.js", "b", "multipart.js", "x.txt"], files.sort());
 });
 
 promise.addErrback(function () {
