@@ -1,4 +1,5 @@
-include("mjsunit.js");
+node.mixin(require("common.js"));
+tcp = require("/tcp.js");
 
 
 var tests_run = 0;
@@ -8,7 +9,7 @@ function pingPongTest (port, host, on_complete) {
   var count = 0;
   var sent_final_ping = false;
 
-  var server = node.tcp.createServer(function (socket) {
+  var server = tcp.createServer(function (socket) {
     assertTrue(socket.remoteAddress !== null);
     assertTrue(socket.remoteAddress !== undefined);
     if (host === "127.0.0.1")
@@ -17,9 +18,11 @@ function pingPongTest (port, host, on_complete) {
       assertEquals(socket.remoteAddress, "127.0.0.1");
 
     socket.setEncoding("utf8");
+    socket.setNoDelay();
     socket.timeout = 0;
 
     socket.addListener("receive", function (data) {
+      puts("server got: " + JSON.stringify(data));
       assertEquals("open", socket.readyState);
       assertTrue(count <= N);
       if (/PING/.exec(data)) {
@@ -40,7 +43,7 @@ function pingPongTest (port, host, on_complete) {
   });
   server.listen(port, host);
 
-  var client = node.tcp.createConnection(port, host);
+  var client = tcp.createConnection(port, host);
 
   client.setEncoding("utf8");
 
